@@ -128,6 +128,8 @@ freqCalc <- function(x, keyVars, w=NULL, alpha=1) {
   # split in complete and non-complete
   dat_without_na <- na.omit(agg, cols=keyVars_n)
   dat_with_na <- na.omit(agg, cols=keyVars_n, invert=TRUE)
+  if (nrow(dat_without_na) == 0)
+    dat_with_na <- copy(agg)
 
   # special treatment - one key variable containing missings
   if (length(keyVars_n)==1) {
@@ -163,7 +165,7 @@ freqCalc <- function(x, keyVars, w=NULL, alpha=1) {
 
   un[,keyid:=.I]
   naind[,sortvar:=.I]
-  naind <- merge(naind, un, by=keyVars_n)
+  naind <- merge(naind, un, by=keyVars_n, suffixes = c("", ".y"))
 
   # we need to resort so that later in the look we can use 'keyid' to set the key
   # for the complete dataset
